<?php

namespace FiloBlu\ExtInventory\Plugin\Api\OrderItemRepositoryInterface;

use Exception;
use FiloBlu\ExtInventory\Api\Data\InventoryFbOrderItemSourceInterfaceFactory;
use FiloBlu\ExtInventory\Api\InventoryFbOrderItemSourceRepositoryInterface;
use FiloBlu\ExtInventory\Helper\Data;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Sales\Api\Data\OrderItemExtensionInterfaceFactory;
use Magento\Sales\Api\Data\OrderItemSearchResultInterface;
use Magento\Sales\Api\OrderItemRepositoryInterface;

/**
 * Class AfterGetList
 * @package FiloBlu\ExtInventory\Plugin\Api\OrderItemRepositoryInterface
 */
class AfterGetList
{

    /**
     * @var InventoryFbOrderItemSourceRepositoryInterface
     */
    private $inventoryFbOrderItemSourceRepository;

    /**
     * @var OrderItemExtensionInterfaceFactory
     */
    private $orderItemExtensionInterfaceFactory;
    /**
     * @var InventoryFbOrderItemSourceInterfaceFactory
     */
    private $fbOrderItemSourceInterfaceFactory;
    /**
     * @var SearchCriteriaBuilder
     */
    private $searchCriteriaBuilder;
    /**
     * @var Data
     */
    private $helperData;

    /**
     * OrderItemRepositoryInterface constructor.
     * @param InventoryFbOrderItemSourceRepositoryInterface $inventoryFbOrderItemSourceRepository
     * @param OrderItemExtensionInterfaceFactory $orderItemExtensionInterfaceFactory
     * @param InventoryFbOrderItemSourceInterfaceFactory $fbOrderItemSourceInterfaceFactory
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param Data $helperData
     */
    public function __construct(
        InventoryFbOrderItemSourceRepositoryInterface $inventoryFbOrderItemSourceRepository,
        OrderItemExtensionInterfaceFactory $orderItemExtensionInterfaceFactory,
        InventoryFbOrderItemSourceInterfaceFactory $fbOrderItemSourceInterfaceFactory,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        Data $helperData
    )
    {
        $this->inventoryFbOrderItemSourceRepository = $inventoryFbOrderItemSourceRepository;
        $this->orderItemExtensionInterfaceFactory = $orderItemExtensionInterfaceFactory;
        $this->fbOrderItemSourceInterfaceFactory = $fbOrderItemSourceInterfaceFactory;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->helperData = $helperData;
    }

    /**
     * @param OrderItemRepositoryInterface $subject
     * @param OrderItemSearchResultInterface $searchCriteria
     * @return OrderItemSearchResultInterface
     */
    public function afterGetList(OrderItemRepositoryInterface $subject, OrderItemSearchResultInterface $searchCriteria)
    {
        $inventoryType = null;
        foreach ($searchCriteria->getItems() as $entity) {
            $extensionAttributes = $entity->getExtensionAttributes();
            if ($extensionAttributes === null) {
                $extensionAttributes = $this->orderItemExtensionInterfaceFactory->create();
            }

            if ($inventoryType === null) {
                $inventoryType = $this->helperData->getInventoryType($entity->getStoreId());
            }
            $extensionAttributes->setFilobluInventoryType($inventoryType);

            try {
                $this->searchCriteriaBuilder->addFilter('order_item_id', $entity->getItemId());
                $filoBluOrderItem = $this->inventoryFbOrderItemSourceRepository->getList($this->searchCriteriaBuilder->addFilter('order_item_id', $entity->getItemId())->create());
                $extensionAttributes->setFilobluOrderItem($filoBluOrderItem->getItems());
            } catch (Exception $exception) {
                $extensionAttributes->setFilobluOrderItem($this->fbOrderItemSourceInterfaceFactory->create());
            }
            $entity->setExtensionAttributes($extensionAttributes);
        }
        return $searchCriteria;
    }
}
